package com.lcfarm.flutter.amap.location.flutter_amap_location_plugin;

import android.app.Activity;
import android.util.Log;

import com.amap.api.location.AMapLocation;
import com.amap.api.location.AMapLocationClient;
import com.amap.api.location.AMapLocationClientOption;
import com.amap.api.location.AMapLocationListener;
import com.amap.api.location.AMapLocationQualityReport;

import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * FlutterAmapLocationPlugin
 */
public class FlutterAmapLocationPlugin implements MethodCallHandler {

    private Registrar registrar;
    private MethodChannel channel;


    public FlutterAmapLocationPlugin(Registrar registrar, MethodChannel channel) {
        this.registrar = registrar;
        this.channel = channel;
    }


    private Activity getApplicationContext() {
        return registrar.activity();
    }

    //定位对象
    private AMapLocationClient locationClient;
    //定位属性
    private AMapLocationClientOption locationOption;

    /**
     * Plugin registration.
     */
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "flutter_amap_location_plugin");
        channel.setMethodCallHandler(new FlutterAmapLocationPlugin(registrar, channel));
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        String method = call.method;
        if ("startup".equals(method)) {
            //初始化
            result.success(startup((Map) call.arguments));
        } else if ("shutdown".equals(method)) {
            //销毁
            result.success(shutdown());
        } else if ("getLocation".equals(method)) {
            //获取定位信息，单次
            boolean needAddress = (boolean) call.arguments;
            getLocation(needAddress, result);
        } else if ("startLocation".equals(method)) {
            //启动定位,如果还没有启动，那么返回false
            result.success(startLocation());
        } else if ("stopLocation".equals(method)) {
            //停止定位
            result.success(stopLocation());
        } else if ("updateOption".equals(method)) {
            //是新定位属性
            result.success(updateOption((Map) call.arguments));
        } else if ("setApiKey".equals(method)) {
            //设置 api key，android 无提供代码 api,在 mainfest 里面配置，build 文件中设置
            result.success(false);
        } else {
            result.notImplemented();
        }
    }

    /**
     * 取得定位信息，单次定位
     *
     * @param result
     * @return
     */
    private boolean getLocation(boolean needAddress, final Result result) {
        //没有初始化定位
        if (locationClient == null) {
            return false;
        }
        locationOption.setNeedAddress(needAddress);
        locationOption.setOnceLocation(true);
        locationClient.setLocationOption(locationOption);

        locationClient.setLocationListener(new AMapLocationListener() {
            @Override
            public void onLocationChanged(AMapLocation aMapLocation) {
                result.success(resultToMap(aMapLocation));
                stopLocation();
            }
        });
        locationClient.startLocation();

        return true;
    }

    /**
     * 开始定位
     *
     * @return
     */
    private boolean startLocation() {
        //没有初始化定位
        if (locationClient == null) {
            return false;
        }

        // 设置定位监听
        locationClient.setLocationListener(locationListener);
        // 启动定位
        locationClient.startLocation();
        return true;
    }

    /**
     * 开始定位
     *
     * @return
     */
    private boolean stopLocation() {
        //没有初始化定位
        if (locationClient == null) {
            return false;
        }
        locationClient.stopLocation();
        return true;
    }


    /**
     * 初始化定位
     *
     * @param arguments
     * @return
     */
    private boolean startup(Map arguments) {
        if (locationClient != null) {
            return false;
        }
        //初始化client
        locationClient = new AMapLocationClient(this.getApplicationContext());
        locationOption = parseOptions(arguments);
        //设置定位参数
        locationClient.setLocationOption(locationOption);
        return true;
    }


    /**
     * 销毁定位
     *
     * @return
     */
    private boolean shutdown() {
        if (null == locationClient) {
            return false;
        }
        /**
         * 如果AMapLocationClient是在当前Activity实例化的，
         * 在Activity的onDestroy中一定要执行AMapLocationClient的onDestroy
         */
        locationClient.onDestroy();
        locationClient = null;
        locationOption = null;
        return true;
    }


    /**
     * 更新定位配置属性
     *
     * @param arguments
     * @return
     */
    private boolean updateOption(Map arguments) {
        //没有初始化定位
        if (locationClient == null) {
            return false;
        }
        locationOption = parseOptions(arguments);
        //设置定位参数
        locationClient.setLocationOption(locationOption);
        return true;
    }


    /**
     * 定位监听
     */
    AMapLocationListener locationListener = new AMapLocationListener() {
        @Override
        public void onLocationChanged(AMapLocation aMapLocation) {
            if (channel == null) return;
            channel.invokeMethod("updateLocation", resultToMap(aMapLocation));
        }
    };


    /**
     * 定位转换成 map,传递到 flutter
     *
     * @param aMapLocation
     * @return
     */
    private Map resultToMap(AMapLocation aMapLocation) {
        Map map = new HashMap();
        if (aMapLocation != null) {
            //定位成功
            if (aMapLocation.getErrorCode() == 0) {
                map.put("success", true);
                //定位类型
                map.put("locationType", aMapLocation.getLocationType());
                //精度
                map.put("accuracy", aMapLocation.getAccuracy());
                //经度
                map.put("longitude", aMapLocation.getLongitude());
                //纬度
                map.put("latitude", aMapLocation.getLatitude());
                //提供者
                map.put("provider", aMapLocation.getProvider());

                map.put("altitude", aMapLocation.getAltitude());
                //定位速度
                map.put("speed", aMapLocation.getSpeed());
                //定位时间
                map.put("timestamp", (double) aMapLocation.getTime() / 1000);
                //地址
                map.put("address", aMapLocation.getAddress());
                //国家
                map.put("country", aMapLocation.getCountry());
                //省
                map.put("province", aMapLocation.getProvince());
                //市
                map.put("city", aMapLocation.getCity());
                //区
                map.put("district", aMapLocation.getDistrict());
                //城市编码
                map.put("cityCode", aMapLocation.getCityCode());
                //区域码
                map.put("adCode", aMapLocation.getAdCode());
                //街道
                map.put("street", aMapLocation.getStreet());
                //街道编码
                map.put("number", aMapLocation.getStreetNum());
                //兴趣点
                map.put("poiName", aMapLocation.getPoiName());

            } else {
                //错误码
                map.put("errorCode", aMapLocation.getErrorCode());
                //错误信息
                map.put("description", aMapLocation.getErrorInfo());
                map.put("success", false);
            }
            //wifi 开关
            map.put("isWifiAble", aMapLocation.getLocationQualityReport().isWifiAble());
            //GPS状态
            map.put("gpsStatus", aMapLocation.getLocationQualityReport().getGPSStatus());
            //GPS星数
            map.put("gpsSatellites", aMapLocation.getLocationQualityReport().getGPSStatus());
            //网络类型
            map.put("networkType", aMapLocation.getLocationQualityReport().getNetworkType());
            //网络耗时
            map.put("netUseTime", aMapLocation.getLocationQualityReport().getNetUseTime());
        }
        return map;
    }

    /**
     * 默认的定位参数，持续定位
     */
    private AMapLocationClientOption getDefaultOption() {
        AMapLocationClientOption mOption = new AMapLocationClientOption();
        mOption.setLocationMode(AMapLocationClientOption.AMapLocationMode.Hight_Accuracy);//可选，设置定位模式，可选的模式有高精度、仅设备、仅网络。默认为高精度模式
        mOption.setGpsFirst(false);//可选，设置是否gps优先，只在高精度模式下有效。默认关闭
        mOption.setHttpTimeOut(30000);//可选，设置网络请求超时时间。默认为30秒。在仅设备模式下无效
        mOption.setInterval(2000);//可选，设置定位间隔。默认为2秒
        mOption.setNeedAddress(true);//可选，设置是否返回逆地理地址信息。默认是true
        mOption.setOnceLocation(false);//可选，设置是否单次定位。默认是false
        mOption.setOnceLocationLatest(false);//可选，设置是否等待wifi刷新，默认为false.如果设置为true,会自动变为单次定位，持续定位时不要使用
        AMapLocationClientOption.setLocationProtocol(AMapLocationClientOption.AMapLocationProtocol.HTTP);//可选， 设置网络请求的协议。可选HTTP或者HTTPS。默认为HTTP
        mOption.setSensorEnable(false);//可选，设置是否使用传感器。默认是false
        mOption.setWifiScan(true); //可选，设置是否开启wifi扫描。默认为true，如果设置为false会同时停止主动刷新，停止以后完全依赖于系统刷新，定位位置可能存在误差
        mOption.setLocationCacheEnable(true); //可选，设置是否使用缓存定位，默认为true
        mOption.setGeoLanguage(AMapLocationClientOption.GeoLanguage.DEFAULT);//可选，设置逆地理信息的语言，默认值为默认语言（根据所在地区选择语言）
        return mOption;
    }


    /**
     * 解析定位参数
     *
     * @param arguments
     * @return
     */
    private AMapLocationClientOption parseOptions(Map arguments) {
        if (arguments == null) {
            return getDefaultOption();
        }
        AMapLocationClientOption option = new AMapLocationClientOption();
        option.setLocationMode(AMapLocationClientOption.AMapLocationMode.valueOf((String) arguments.get("locationMode")));//可选，设置定位模式，可选的模式有高精度、仅设备、仅网络。默认为高精度模式
        option.setGpsFirst((Boolean) arguments.get("gpsFirst"));//可选，设置是否gps优先，只在高精度模式下有效。默认关闭
        option.setHttpTimeOut((Integer) arguments.get("httpTimeOut"));//可选，设置网络请求超时时间。默认为30秒。在仅设备模式下无效
        option.setInterval((Integer) arguments.get("interval"));//可选，设置定位间隔。默认为2秒
        option.setNeedAddress((Boolean) arguments.get("needsAddress"));//可选，设置是否返回逆地理地址信息。默认是true
        option.setOnceLocation((Boolean) arguments.get("onceLocation"));//可选，设置是否单次定位。默认是false
        option.setOnceLocationLatest((Boolean) arguments.get("onceLocationLatest"));//可选，设置是否等待wifi刷新，默认为false.如果设置为true,会自动变为单次定位，持续定位时不要使用
        AMapLocationClientOption.setLocationProtocol(AMapLocationClientOption.AMapLocationProtocol.valueOf((String) arguments.get("locationProtocal")));//可选， 设置网络请求的协议。可选HTTP或者HTTPS。默认为HTTP
        option.setSensorEnable((Boolean) arguments.get("sensorEnable"));//可选，设置是否使用传感器。默认是false
        option.setWifiScan((Boolean) arguments.get("wifiScan")); //可选，设置是否开启wifi扫描。默认为true，如果设置为false会同时停止主动刷新，停止以后完全依赖于系统刷新，定位位置可能存在误差
        option.setLocationCacheEnable((Boolean) arguments.get("locationCacheEnable")); //可选，设置是否使用缓存定位，默认为true
        option.setGeoLanguage(AMapLocationClientOption.GeoLanguage.valueOf((String) arguments.get("geoLanguage")));//可选，设置逆地理信息的语言，默认值为默认语言（根据所在地区选择语言）
        return option;
    }

}
