#import "FlutterAmapLocationPlugin.h"
#import <AMapFoundationKit/AMapFoundationKit.h>
#import <AMapLocationKit/AMapLocationKit.h>

@interface FlutterAmapLocationPlugin () <AMapLocationManagerDelegate>

@property (nonatomic, strong) AMapLocationManager *locationManager;

@property (nonatomic, copy) AMapLocatingCompletionBlock completionBlock;

@property (nonatomic, weak) FlutterMethodChannel* channel;

@end


@implementation FlutterAmapLocationPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel
                                     methodChannelWithName:@"flutter_amap_location_plugin"
                                     binaryMessenger:[registrar messenger]];
    FlutterAmapLocationPlugin* instance = [[FlutterAmapLocationPlugin alloc] init];
    instance.channel=channel;
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    NSString* method=call.method;
    
    if ([@"getPlatformVersion" isEqualToString:method]) {
        result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
    }else if([@"startup" isEqualToString:method]){
        //初始化
        result(@([self startup:call.arguments]));
    } else if([@"shutdown" isEqualToString:method]){
        //销毁p
        result(@([self shutdown]));
    }else if([@"startLocation" isEqualToString:method]){
        //启动定位
        result(@([self startLocation]));
    } else if([@"stopLocation" isEqualToString:method]){
        //停止定位
        result(@([self stopLocation]));
    }else if([@"getLocation" isEqualToString:method]){
        //获取定位信息，单次
        bool needAddress = (bool) call.arguments;
       [self getLocation:result needAddress:needAddress];
    }else if([@"updateOption" isEqualToString:method]){
        //更新定位配置
        result(@([self updateOption:call.arguments]));
    } else if([@"setApiKey" isEqualToString:method]){
        //设置定位 key
        [AMapServices sharedServices].apiKey = call.arguments;
        result(@YES);
    } else {
        result(FlutterMethodNotImplemented);
    }
}

#pragma mark - 初始化定位
- (BOOL) startup:(NSDictionary*) arguments{
    if(self.locationManager) return NO;
    
    self.locationManager = [[AMapLocationManager alloc] init];
    
    [self.locationManager setDelegate:self];
    
    return [self updateOption:arguments];
}

#pragma mark - 销毁定位
- (BOOL) shutdown{
    if(self.locationManager==nil){
        return NO;
    }
    //停止定位
    [self.locationManager stopUpdatingLocation];
    
    [self.locationManager setDelegate:nil];
    
    self.locationManager=nil;
    return YES;
}

#pragma mark - 开始定位
-(BOOL) startLocation{
    if(self.locationManager==nil){
        return NO;
    }
    [self.locationManager startUpdatingLocation];
    return YES;
}

#pragma mark - 停止定位
-(BOOL) stopLocation{
    if(self.locationManager==nil){
        return NO;
    }
    [self.locationManager stopUpdatingLocation];
    return YES;
}

#pragma mark - 获取定位信息
-(BOOL) getLocation:(FlutterResult)result needAddress:(BOOL) needAddress{
    if(self.locationManager==nil){
        return NO;
    }
    
    self.completionBlock = ^(CLLocation *location, AMapLocationReGeocode *reGeoCode, NSError *error)
    {
        if (error != nil && error.code == AMapLocationErrorLocateFailed)
        {
            //定位错误：此时location和regeocode没有返回值，不进行annotation的添加
            NSLog(@"定位错误:{%ld - %@};", (long)error.code, error.localizedDescription);
            
            result(@{ @"code":@(error.code),@"description":error.localizedDescription, @"success":@NO });
            
            return;
        }
        else if (error != nil
                 && (error.code == AMapLocationErrorReGeocodeFailed
                     || error.code == AMapLocationErrorTimeOut
                     || error.code == AMapLocationErrorCannotFindHost
                     || error.code == AMapLocationErrorBadURL
                     || error.code == AMapLocationErrorNotConnectedToInternet
                     || error.code == AMapLocationErrorCannotConnectToHost))
        {
            //逆地理错误：在带逆地理的单次定位中，逆地理过程可能发生错误，此时location有返回值，regeocode无返回值，进行annotation的添加
            NSLog(@"逆地理错误:{%ld - %@};", (long)error.code, error.localizedDescription);
        }
        else if (error != nil && error.code == AMapLocationErrorRiskOfFakeLocation)
        {
            //存在虚拟定位的风险：此时location和regeocode没有返回值，不进行annotation的添加
            NSLog(@"存在虚拟定位的风险:{%ld - %@};", (long)error.code, error.localizedDescription);
            result(@{ @"code":@(error.code),@"description":error.localizedDescription, @"success":@NO });
           return;
        }
        else
        {
            //没有错误：location有返回值，regeocode是否有返回值取决于是否进行逆地理操作，进行annotation的添加
        }
        
        NSMutableDictionary* md = [[NSMutableDictionary alloc]initWithDictionary: [FlutterAmapLocationPlugin location2map:location]  ];
        if (reGeoCode)
        {
            [md addEntriesFromDictionary:[FlutterAmapLocationPlugin reGeoCode2map:reGeoCode]];
            md[@"success"] = @YES;
        }
        else
        {
            md[@"errorCode"]=@(error.code);
            md[@"description"]=error.localizedDescription;
            md[@"success"] = @YES;
        }
        
        result(md);
        
        
    };
    
    //进行单次带逆地理定位请求
    [self.locationManager requestLocationWithReGeocode:needAddress completionBlock:self.completionBlock];
    
    return YES;
}


#pragma mark - 更新定位配置
-(BOOL) updateOption:(NSDictionary*) arguments{
    if(arguments==nil){
        return NO;
    }
    //设置定位精度
    [self.locationManager setDesiredAccuracy: [self getDesiredAccuracy:arguments[@"desiredAccuracy"]]];
    
    //设置不允许系统暂停定位
    [self.locationManager setPausesLocationUpdatesAutomatically:[arguments[@"pausesLocationUpdatesAutomatically"] boolValue] ];
    
    //设置允许在后台定位
    [self.locationManager setAllowsBackgroundLocationUpdates:[arguments[@"allowsBackgroundLocationUpdates"] boolValue]];
    
    
    //设置定位超时时间
    [self.locationManager setLocationTimeout:[arguments[@"lLocationTimeout"] boolValue]];
    
    //设置逆地理超时时间
    [self.locationManager setReGeocodeTimeout:[arguments[@"reGeocodeTimeout"] boolValue]];
    
    //设置允许连续定位逆地理
    [self.locationManager setLocatingWithReGeocode:[arguments[@"locatingWithReGeocode"] boolValue]];
    
    return YES;
}



#pragma mark - AMapLocationManager Delegate

//定位权限状态改变时回调函数
- (void)amapLocationManager:(AMapLocationManager *)manager doRequireLocationAuth:(CLLocationManager *)locationManager
{
    [locationManager requestAlwaysAuthorization];
}

//当定位发生错误时，会调用代理的此方法。
- (void)amapLocationManager:(AMapLocationManager *)manager didFailWithError:(NSError *)error
{
    NSLog(@"%s, amapLocationManager = %@, error = %@", __func__, [manager class], error);
    
    if (error.code == AMapLocationErrorRiskOfFakeLocation) {
        NSLog(@"存在虚拟定位的风险:{%ld - %@};", (long)error.code, error.userInfo);
        
        //存在虚拟定位的风险的定位结果
        __unused CLLocation *riskyLocateResult = [error.userInfo objectForKey:@"AMapLocationRiskyLocateResult"];
        //存在外接的辅助定位设备
        __unused NSDictionary *externalAccressory = [error.userInfo objectForKey:@"AMapLocationAccessoryInfo"];
    }
    
    [self.channel invokeMethod:@"updateLocation" arguments:@{ @"errorCode":@(error.code),@"description":error.localizedDescription,@"success":@NO }];
}

//连续定位回调函数.注意：如果实现了本方法，则定位信息不会通过amapLocationManager:didUpdateLocation:方法回调。
- (void)amapLocationManager:(AMapLocationManager *)manager didUpdateLocation:(CLLocation *)location reGeocode:(AMapLocationReGeocode *)reGeocode
{
    NSLog(@"location:{lat:%f; lon:%f; accuracy:%f; reGeocode:%@}", location.coordinate.latitude, location.coordinate.longitude, location.horizontalAccuracy, reGeocode.formattedAddress);
    
    NSMutableDictionary* md = [[NSMutableDictionary alloc]initWithDictionary: [FlutterAmapLocationPlugin location2map:location]  ];
    if (reGeocode)
    {
        [md addEntriesFromDictionary:[FlutterAmapLocationPlugin reGeoCode2map:reGeocode]];
        
    }
    md[@"success"] = @YES;
    
    [self.channel invokeMethod:@"updateLocation" arguments:md];
    
}




//从flutter 传地来参数中解析
-(double) getDesiredAccuracy:(NSString*) accuracy{
    if([@"kCLLocationAccuracyBest" isEqualToString:accuracy]){
        return kCLLocationAccuracyBest;
    }else if([@"kCLLocationAccuracyNearestTenMeters" isEqualToString:accuracy]){
        return kCLLocationAccuracyNearestTenMeters;
    }else if([@"kCLLocationAccuracyHundredMeters" isEqualToString:accuracy]){
        return kCLLocationAccuracyHundredMeters;
    }
    else if([@"kCLLocationAccuracyKilometer" isEqualToString:accuracy]){
        return kCLLocationAccuracyKilometer;
    }
    else{
        return kCLLocationAccuracyThreeKilometers;
    }
}



+(id)checkNull:(NSObject*)value{
    return value == nil ? [NSNull null] : value;
}


+(NSDictionary*)reGeoCode2map:(AMapLocationReGeocode *)reGeoCode{
    return @{@"address":reGeoCode.formattedAddress,
             @"country":reGeoCode.country,
             @"province":reGeoCode.province,
             @"city":reGeoCode.city,
             @"district":reGeoCode.district,
             @"cityCode":reGeoCode.citycode,
             @"adCode":reGeoCode.adcode,
             @"street":reGeoCode.street,
             @"number":reGeoCode.number,
             @"poiName":[self checkNull : reGeoCode.POIName],
             @"aoiName":[self checkNull :reGeoCode.AOIName],
             };
}

+(NSDictionary*)location2map:(CLLocation *)location{
    
    return @{@"latitude": @(location.coordinate.latitude),
             @"longitude": @(location.coordinate.longitude),
             @"accuracy": @((location.horizontalAccuracy + location.verticalAccuracy)/2),
             @"altitude": @(location.altitude),
             @"speed": @(location.speed),
             @"timestamp": @(location.timestamp.timeIntervalSince1970),};
    
}

@end
